#ifndef SPARSE_H
#define SPARSE_H

// sparse matrix class

#include <iostream>
#include <map>
#include <algorithm>
#include <assert.h>

using namespace std;

typedef int index_t;

template<class K, class V> ostream& dump_map(map<K,V> m, ostream& os)
{
  for (typename map<K,V>::iterator i = m.begin(); i != m.end(); ++i)
    os << i->first << " => " << i->second << endl;
  return os;
}

template<class N> class sparse_matrix {
  index_t rows;
  index_t cols;
  
  struct pos {
    index_t i, j;
    pos(index_t i_, index_t j_) : i (i_), j (j_) { }
    ~pos() { }
    bool operator< (const pos& p) const { return (i < p.i) || (i == p.i && j < p.j); }
  };
  typedef map<pos,N> vals_t;
  vals_t vals;

  bool okpos(const pos& p)
    {
      return (1 <= p.i && p.i <= rows && 1 <= p.j && p.j <= cols);
    }

  friend ostream& operator<< (ostream& os, const pos& p)
    {
      os << '(' << p.i << ',' << p.j << ')';
      return os;
    }

 public:
  sparse_matrix() : rows(0), cols(0) { }

  sparse_matrix(index_t rows_, index_t cols_) : rows(rows_), cols(cols_) { }

  sparse_matrix(N c) : rows(1), cols(1) { vals[pos(1,1)] = c; }
  // default copy constructor works
  ~sparse_matrix() { }

  index_t get_rows() const { return rows; }
  index_t get_cols() const { return cols; }

  bool square() const { return get_rows() == get_cols(); }

  void swap (sparse_matrix<N>& m) {
    swap(rows, m.rows);
    swap(cols, m.cols);
    swap(vals, m.vals); // fast
  }

  void clear (void)
    {
      rows = cols = 0;
      vals.clear();
    }

  void cleanup (void) {
    vals_t newval;
    for (typename vals_t::iterator i = vals.begin(); i != vals.end(); ++i)
      {
	assert(okpos(i->first));
	if (i->second != 0)
	  newval[i->first] = i->second;
      }
    std::swap(vals, newval);
  }

  N operator() (index_t i, index_t j) const {
    typename vals_t::const_iterator it = vals.find(pos(i,j));
    if (it == vals.end())
      return 0;
    else
      return it->second;
  }

  N& operator() (index_t i, index_t j) {
    if (i > rows)
      rows = i;
    if (j > cols)
      cols = j;
    return vals[pos(i,j)];
  }

  void kill_block(index_t ki, index_t kj, index_t krows, index_t kcols) {
    if (ki > rows || kj > cols)
      return;
    vals_t newval;
    for (typename vals_t::iterator it = vals.begin(); it != vals.end(); ++it)
      {
	assert(okpos(it->first));
	index_t i = it->first.i, j = it->first.j;
	if ((i < ki || i >= ki + krows) || (j < kj || j >= kj + kcols))
	  newval[it->first] = it->second;
      }
    std::swap(vals, newval);
  }

  void insert (const sparse_matrix<N>& m, index_t i, index_t j) {
    assert(&m != this);
    kill_block(i, j, m.rows, m.cols);
    rows = max(rows, i + m.rows - 1);
    cols = max(cols, j + m.cols - 1);
    for (typename vals_t::const_iterator it = m.vals.begin(); it != m.vals.end(); ++it)
      {
	if (it->second != 0)
	  {
	    pos p(it->first.i + (i - 1), it->first.j + (j - 1));
	    vals[p] = it->second;
	  }
      }
  }

  void scale (N c) {
    if (c == 0)
      {
	vals.clear();
	return;
      }
    else if (c == 1)
      return;
    else
      {
	for (typename vals_t::iterator it = vals.begin(); it != vals.end(); ++it)
	  {
	    assert(okpos(it->first));
	    it->second *= c;
	  }
      }
  }
  
  inline const sparse_matrix<N>& operator*= (N c)
    {
      scale(c);
      return *this;
    }

  friend inline sparse_matrix<N> operator* (const sparse_matrix<N> &m, N c)
    {
      sparse_matrix<N> m1(m);
      m1 *= c;
      return m1;
    }
  friend inline sparse_matrix<N> operator* (N c, const sparse_matrix<N> &m)
    {
      return m * c;
    }

  // turn matrix into size x size identity
  void idn(index_t size)
    {
      rows = cols = size;
      vals.clear();
      for (int i = 1; i <= size; i++)
	vals[pos(i,i)] = 1;
    }

  static inline sparse_matrix<N> make_idn(index_t size) {
    sparse_matrix<N> m;
    m.idn(size);
    return m;
  }

  // fill in matrix with a tensor b
  static void tensor (const sparse_matrix<N>& x,
	       const sparse_matrix<N>& y,
	       sparse_matrix<N>& r) {
    assert(&x != &r);
    assert(&y != &r);
    r.clear();
    for (int i = 1; i <= x.rows; i++)
      for (int j = 1; j <= x.cols; j++)
	{
	  r.insert(y * x(i,j), (y.rows * (i-1)) + 1, (y.cols * (j-1)) + 1);
	}
    assert(r.rows == x.rows * y.rows);
    assert(r.cols == x.cols * y.cols);
  }

  // mmm, copying

  static inline sparse_matrix<N> tensor (const sparse_matrix<N>& x,
				  const sparse_matrix<N>& y) {
    sparse_matrix<N> r;
    tensor(x,y,r);
    return r;
  }

  void dumpme (void)
    {
      dump_map(vals, cerr);
    }

#define cout YARRGH
  inline void print_row (index_t i, ostream& os) const {
    for (int j = 1; j < cols; j++)
      os << (*this)(i,j) << ' ';
    os << (*this)(i,cols);
  }

  void print_nicely (ostream& os) const {
    if (rows == 0)
      os << "(empty)" << endl;
    else if (rows == 1)
      {
	os << '(';
	print_row(1, os);
	os << ')' << endl;
      }
    else
      {
	os << '/';
	print_row(1, os);
	os << '\\' << endl;
	for (int i = 2; i < rows; i++)
	  {
	    os << '|';
	    print_row(i, os);
	    os << '|' << endl;
	  }
	os << '\\';
	print_row(rows, os);
	os << '/' << endl;
      }
  }
#undef cout

  void dump_nice () const {
    print_nicely(cerr);
  }

  N trace (void) {
    assert(rows == cols);
    N sum = 0;
    for (int i = 1; i <= rows; i++)
      sum += (*this)(i,i);
    return sum;
  }

  static void mult (const sparse_matrix<N>& a,
		    const sparse_matrix<N>& b,
		    sparse_matrix<N>& r)
    {
      assert(a.get_rows() == b.get_cols());
      assert(&a != &r);
      assert(&b != &r);
      r.clear();
      for (typename vals_t::const_iterator it = a.vals.begin(); it != a.vals.end(); ++it)
	{
	  index_t i = it->first.i, j = it->first.j;
	  for (index_t k = 1; k <= b.get_cols(); k++)
	    {
	      N x = it->second * b(j,k);
	      if (x != 0)
		r(i,k) += x;
	    }
	}
    }
  friend inline sparse_matrix<N> operator* (const sparse_matrix<N>& a,
					    const sparse_matrix<N>& b) {
    sparse_matrix<N> r;
    mult(a,b,r);
    return r;
  }

  sparse_matrix<N>& operator*= (const sparse_matrix<N>& a) {
    sparse_matrix<N> t(*this);
    mult(t, a, *this);
    return *this;
  }

  inline sparse_matrix<N>& operator+= (const sparse_matrix<N>& a) {
    assert(a.get_rows() == rows);
    assert(a.get_cols() == cols);
    for (typename vals_t::const_iterator it = a.vals.begin(); it != a.vals.end(); ++it)
      (*this)(it->first.i, it->first.j) += it->second;
    return *this;
  }

  inline sparse_matrix<N>& operator-= (const sparse_matrix<N>& a) {
    assert(a.get_rows() == rows);
    assert(a.get_cols() == cols);
    for (typename vals_t::const_iterator it = a.vals.begin(); it != a.vals.end(); ++it)
      (*this)(it->first.i, it->first.j) -= it->second;
    return *this;
  }

  friend inline sparse_matrix<N> operator+ (const sparse_matrix<N> a,
					    const sparse_matrix<N> b) {
    sparse_matrix<N> r(a);
    r += b;
    return r;
  }

  friend inline sparse_matrix<N> operator- (const sparse_matrix<N> a,
					    const sparse_matrix<N> b) {
    sparse_matrix<N> r(a);
    r -= b;
    return r;
  }

  void direct_sum (const sparse_matrix<N>& a) {
    insert(a, rows+1, cols+1);
  }

  static sparse_matrix<N> direct_sum (const sparse_matrix<N>& a,
				      const sparse_matrix<N>& b)
    {
      sparse_matrix<N> r(a);
      r.direct_sum(b);
      return r;
    }

  void transpose (void) {
    vals_t v1;
    for (typename vals_t::iterator it = vals.begin(); it != vals.end(); ++it)
      v1[pos(it->first.j, it->first.i)] = it->second;
    std::swap(vals, v1);
    std::swap(rows, cols);
  }

  inline sparse_matrix<N> kill_row_and_col(index_t i, index_t j) const {
    assert(1 <= i && i <= rows && 1 <= j && j <= cols);
    sparse_matrix<N> m;
    m.rows = rows - 1;
    m.cols = cols - 1;
    for (typename vals_t::const_iterator it = vals.begin(); it != vals.end(); ++it)
      {
	index_t ii = it->first.i, jj=it->first.j;
	if (ii == i || jj == j)
	  continue;
	if (ii > i) ii--;
	if (jj > j) jj--;
	m(ii,jj) = it->second;
      }
    return m;
  }

  N det (void) const {
    assert(square());
    if (rows == 1 && cols == 1)
      return (*this)(1,1);
    N sgn = 1;
    N d = 0;
    for (index_t i = 1; i <= rows;  i++)
      {
	if ((*this)(i,1) != 0)
	  d += sgn * (*this)(i,1) * kill_row_and_col(i,1).det();
	sgn = -sgn;
      }
    return d;
  }
	  
  inline bool singular () const {
    return det() == 0;
  }
  
};

template<class N> void swap(sparse_matrix<N>& a, sparse_matrix<N>& b)
{
  a.swap(b);
}

#endif
